﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Kona.Model;
using Kona.Infrastructure;
using System.Data;
using System.Data.Common;

namespace Kona.SimpleSqlRepository {
    
    public class SimpleProductRepository:IProductRepository {

        string connectionStringName = "KonaConnectionString";
        internal Product LoadProduct(DbDataReader rdr) {
            return new Product(
                ProductsTable.ReadSKU(rdr),
                ProductsTable.ReadProductName(rdr),
                ProductsTable.ReadIsTaxable(rdr),
                ProductsTable.ReadWeightInPounds(rdr),
                ProductsTable.ReadAmountOnHand(rdr),
                ProductsTable.ReadDateAvailable(rdr),
                ProductsTable.ReadAllowBackOrder(rdr),
                ProductsTable.ReadAllowPreOrder(rdr))
            {

                DefaultImage = new Image(
                    ProductsTable.ReadDefaultImageFile(rdr),
                    ProductsTable.ReadDefaultImageFile(rdr)),
                EstimatedDelivery = ProductsTable.ReadEstimatedDelivery(rdr),
                Price = ProductsTable.ReadBasePrice(rdr)
            };
        }

        public Product GetProduct(string sku) {
            Product result=null;
            var batch = new BatchSql();
            batch.Append(ProductsTable.Select().Where(ProductsTable.ColumnsQualified.SKU, sku));
            
            //desciptors
            batch.Append(ProductDescriptorsTable.Select()
                .Where(ProductDescriptorsTable.ColumnsQualified.SKU, sku)
                .And(ProductDescriptorsTable.ColumnsQualified.LanguageCode,System.Globalization.CultureInfo.CurrentCulture.TwoLetterISOLanguageName));
            
            //images
            batch.Append(ProductImagesTable.Select().Where(ProductImagesTable.ColumnsQualified.SKU, sku));

            
            //cross sells
            batch.Append(ProductsTable.Select().Add("\r\nWHERE sku in (SELECT CrossSku FROM Products_CrossSell WHERE Products_CrossSell.SKU=@p0)"));
            //TODO: Fix this
            
            
            //related
            batch.Append(ProductsTable.Select().Add("\r\nWHERE sku in (SELECT RelatedSKu FROM Products_Related WHERE Products_Related.SKU=@p0)"));

            var cmd = batch.BuildCommand(connectionStringName);
            
            using(var rdr=cmd.ExecuteReader(CommandBehavior.CloseConnection)){
                
                if(rdr.Read()){
                    result = LoadProduct(rdr);
                }
                if (result != null) {
                    //desciptors
                    result.Descriptors = new List<ProductDescriptor>();
                    if (rdr.NextResult()) {
                        while (rdr.Read()) {
                            result.Descriptors.Add(new ProductDescriptor(
                                ProductDescriptorsTable.ReadTitle(rdr),
                                ProductDescriptorsTable.ReadBody(rdr)
                                ));
                        }
                    }

                    //images
                    result.Images = new List<Image>();
                    if (rdr.NextResult()) {
                        while (rdr.Read()) {
                            result.Images.Add(new Image(
                                ProductImagesTable.ReadThumbUrl(rdr),
                                ProductImagesTable.ReadFullImageUrl(rdr)));
                        }
                    }

                    //cross sells
                    result.CrossSells = new List<Product>();
                    if (rdr.NextResult()) {
                        while (rdr.Read()) {
                            result.CrossSells.Add(LoadProduct(rdr));
                        }
                    }
                    //related
                    result.RelatedProducts = new List<Product>();
                    if (rdr.NextResult()) {
                        while (rdr.Read()) {
                            result.RelatedProducts.Add(LoadProduct(rdr));
                        }
                    }
                }
            }

            return result;

        }

        public IList<Product> GetProducts() {
            var sql = ProductsTable.Select();
            var cmd = sql.BuildCommand();
            List<Product> result = new List<Product>();


            using (var rdr = cmd.ExecuteReader(CommandBehavior.CloseConnection)) {

                while (rdr.Read()) {
                    result.Add(LoadProduct(rdr));
                }
            }
            return result;

        }

    }
}
