using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using Kona.Model;
using Kona.Infrastructure;
using System.Text;
using System.IO;

namespace Kona.Web.Controllers
{

    [ValidateInput(false)]
    public class PageController : KonaController
    {
        //
        // GET: /Page/
        IProductRepository _productRepository;
        ICMSRepository _cmsRepository;
        ICustomerRepository _customerRepository;
        IObjectStore _objectStore;

        public PageController(ICustomerRepository customerRepository,
            ICMSRepository cmsRepository,
            IObjectStore objectStore,
            IProductRepository productRepository) : base(customerRepository,objectStore,cmsRepository) {
            _productRepository = productRepository;
            _cmsRepository = cmsRepository;
            _objectStore = objectStore;
            _customerRepository = customerRepository;


        }

        public ActionResult New() {

            //pull a list of drafts
            var drafts = _cmsRepository.GetPages(PublishStatus.Draft);

            //create a new GUID for this page
            return View("NewPage",drafts);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(Page p) {
            if (String.IsNullOrEmpty(p.Title)) {
                this.ModelState.AddModelError("notitle", "The page must have a title");
                return View("NewPage");
            } else {
                p.PageID = Guid.NewGuid();
                p.LanguageCode = System.Globalization.CultureInfo.CurrentCulture.TwoLetterISOLanguageName;
                p.Slug = p.Title.CreateSlug();
                p.CreatedBy = User.Identity.Name;
                p.ModifiedBy = User.Identity.Name;
                p.Url = p.Slug;

                if (ModelState.IsValid) {
                    _cmsRepository.Create(p);
                }
                return RedirectToAction("Edit", new { id = p.PageID });
            }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SaveProductImage() {

            Guid widgetID=new Guid(Request.Form["widgetid"]);
            Guid pageID = new Guid(Request.Form["pageid"]);

            if (Request.Files.Count > 0) {
                var item = Request.Files[0] as HttpPostedFileBase;
                string fileExtension=Path.GetExtension(item.FileName);
                string fileName=Path.GetFileName(item.FileName);
                string filePath=Path.Combine(Server.MapPath("~/content/images"),fileName);
                item.SaveAs(filePath);

                //now that the file's uploaded, save the file name as the body of the widget
                var widget = _cmsRepository.GetWidget(widgetID);
                widget.Body = fileName;
                widget.ModifiedOn = DateTime.Now;
                _cmsRepository.SaveWidget(widget);


            }
            return RedirectToAction("Edit", new { id = pageID });
        }


        public string SkuList() {
            var pref = Request.QueryString["q"] ?? "";
            List<Product> products = _productRepository.GetProducts().ToList();
            products = products.Where(x => x.SKU.StartsWith(pref, StringComparison.CurrentCultureIgnoreCase)).OrderBy(x => x.SKU).ToList();
            StringBuilder sb = new StringBuilder();
            products.ForEach(x => sb.Append(x.SKU+"\r\n"));
            return sb.ToString();
        }

        [AcceptVerbs(HttpVerbs.Get)]
        public ActionResult Edit(string id) {
             //pull the page and widgets
            var pg = _cmsRepository.GetPage(new Guid(id));
            if (pg != null) {
                return View("EditPage",pg);
            } else {
                return RedirectToAction("Create");
            }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Edit(Page page) {

            if (page != null) {
                page.ModifiedBy = User.Identity.Name;
                page.ModifiedOn = DateTime.Now;
                page.LanguageCode = System.Globalization.CultureInfo.CurrentCulture.TwoLetterISOLanguageName;
                page.Slug = page.Title.CreateSlug();
                page.Url = "";

                _cmsRepository.Update(page);

                return View("EditPage", page);
            } else {
                return RedirectToAction("Create");
            }
        }
        public ActionResult Preview(string id) {
            Guid pageID = new Guid(id);
            this.CurrentPage = _cmsRepository.GetPage(pageID);
            return View("Preview");
        }

        public ActionResult SortPages() {
            if (Request.Form["pageid"] != null) {
                //pull out all pageid's
                var pages = Request.Form.GetValues("pageid").Where(x => !String.IsNullOrEmpty(x)).ToArray();
                _cmsRepository.OrderPages(pages);
            }
            return new EmptyResult();
        }

        public ActionResult List() {
            var pages = _cmsRepository.GetPages();
            return View("Pages", pages);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SetStatus(string id, string status) {

            Guid pageID = new Guid(id);
            ActionResult result;
            var pg = _cmsRepository.GetPage(pageID);
            
            if (status == "Publish") {
                //pull the page
                pg.Status = PublishStatus.Published;
                _cmsRepository.Update(pg);
                return RedirectToAction("Index", "Home", new { slug = pg.Slug });
            } else if (status == "Take Offline") {
                //offline
                pg.Status = PublishStatus.Offline;
                _cmsRepository.Update(pg);
                return RedirectToAction("Edit", "Page", new { id = pg.PageID.ToString() });

            } else if (status == "Delete") {
                _cmsRepository.DeletePage(pageID);
                return RedirectToAction("Index", "Home");
            } else {
                return RedirectToAction("Edit", "Page", new { id = pg.PageID.ToString() });

            }


        }


        public ActionResult Editor() {
            return View("TextEditor");
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult RemoveWidget(string id) {
            if (!String.IsNullOrEmpty(id)) {
                _cmsRepository.DeleteWidget(new Guid(id));
            }
            return new EmptyResult();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult AddWidget(Widget widget) {
            widget.ID = Guid.NewGuid();
            widget.EditorName = widget.ViewName + "Editor";
            widget.LanguageCode = System.Globalization.CultureInfo.CurrentCulture.TwoLetterISOLanguageName;
            widget.Title = widget.Title ?? "";
            widget.Body = widget.Body ?? "";
            widget.IsTyped = widget.ViewName.Contains("Product");
            //save em
            _cmsRepository.SaveWidget(widget);
            return Json(widget);
        }

        public ActionResult RenderWidget(string id) {
            Guid widgetID = new Guid(id);
            IWidget widget = _cmsRepository.GetWidget(widgetID);
            if (widget is Widget<IList<Product>>) {
                var typedWidget = widget as Widget<IList<Product>>;
                return View(typedWidget.EditorName,typedWidget);
            }
            else {
                return View(widget.EditorName,widget);
            }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SaveWidget(Widget widget) {

            //pull the widget to keep the data current
            IWidget existing = _cmsRepository.GetWidget(widget.ID);

            existing.ModifiedOn = DateTime.Now;
            existing.ModifiedBy = User.Identity.Name;
            existing.Title = widget.Title;
            if (!string.IsNullOrEmpty(widget.Body))
                existing.Body = widget.Body;
            _cmsRepository.SaveWidget(existing);
            return new EmptyResult();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SaveProductWidget() {
            var widgetID = Request.Form["widgetid"] ?? "";
            if (!string.IsNullOrEmpty(widgetID)) {
                //pull out all sku's
                var skus = Request.Form.GetValues("sku");
                _cmsRepository.SaveProductWidget(new Guid(widgetID), skus);

            }
            return new EmptyResult();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult OrderWidgets() {
            if (Request.Form["widgetid"]!=null) {
                //pull out all widgetid's
                var widgets = Request.Form.GetValues("widgetid").Where(x=>!String.IsNullOrEmpty(x)).ToArray();
                _cmsRepository.OrderWidgets(widgets);
            }
            return new EmptyResult();
        }

        public ActionResult GetProduct(string id) {
            //this is for AJAX calls
            var product = _productRepository.GetProduct(id);
            if (product != null) {

                JsonProduct jsonProduct = new JsonProduct();
                jsonProduct.SKU = product.SKU;
                jsonProduct.Photo = product.DefaultImage.ThumbnailPhoto;
                jsonProduct.ProductName = product.Name;
                jsonProduct.Price = product.Price.ToString("C");

                return Json(jsonProduct);
            } else {
                return new EmptyResult();
            }
        }

    }
    public class JsonProduct {
        public string Photo { get; set; }
        public string ProductName { get; set; }
        public string Price { get; set; }
        public string SKU { get; set; }
    }
}
